@@ -35,6 +35,7 @@ from google.api_core.exceptions import NotFound
 from google.cloud._helpers import _datetime_to_pb_timestamp
 from google.cloud._helpers import UTC
 from google.cloud import firestore_v1 as firestore
+from google.cloud.firestore_v1.base_query import FieldFilter, And, Or
 
 from tests.system.test__helpers import (
     FIRESTORE_CREDS,
@@ -586,14 +587,29 @@ async def query_docs(client):
 @pytest_asyncio.fixture
 async def async_query(query_docs):
     collection, stored, allowed_vals = query_docs
-    query = collection.where("a", "==", 1)
+    query = collection.where(filter=FieldFilter("a", "==", 1))
 
     return query
 
 
+async def test_query_stream_legacy_where(query_docs):
+    """Assert the legacy code still works and returns value, and shows UserWarning"""
+    collection, stored, allowed_vals = query_docs
+    with pytest.warns(
+        UserWarning,
+        match="Detected filter using positional arguments",
+    ):
+        query = collection.where("a", "==", 1)
+        values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
+        assert len(values) == len(allowed_vals)
+        for key, value in values.items():
+            assert stored[key] == value
+            assert value["a"] == 1
+
+
 async def test_query_stream_w_simple_field_eq_op(query_docs):
     collection, stored, allowed_vals = query_docs
-    query = collection.where("a", "==", 1)
+    query = collection.where(filter=FieldFilter("a", "==", 1))
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
     for key, value in values.items():
@@ -603,7 +619,7 @@ async def test_query_stream_w_simple_field_eq_op(query_docs):
 
 async def test_query_stream_w_simple_field_array_contains_op(query_docs):
     collection, stored, allowed_vals = query_docs
-    query = collection.where("c", "array_contains", 1)
+    query = collection.where(filter=FieldFilter("c", "array_contains", 1))
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
     for key, value in values.items():
@@ -614,7 +630,7 @@ async def test_query_stream_w_simple_field_array_contains_op(query_docs):
 async def test_query_stream_w_simple_field_in_op(query_docs):
     collection, stored, allowed_vals = query_docs
     num_vals = len(allowed_vals)
-    query = collection.where("a", "in", [1, num_vals + 100])
+    query = collection.where(filter=FieldFilter("a", "in", [1, num_vals + 100]))
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
     for key, value in values.items():
@@ -625,7 +641,9 @@ async def test_query_stream_w_simple_field_in_op(query_docs):
 async def test_query_stream_w_simple_field_array_contains_any_op(query_docs):
     collection, stored, allowed_vals = query_docs
     num_vals = len(allowed_vals)
-    query = collection.where("c", "array_contains_any", [1, num_vals * 200])
+    query = collection.where(
+        filter=FieldFilter("c", "array_contains_any", [1, num_vals * 200])
+    )
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
     for key, value in values.items():
@@ -648,7 +666,7 @@ async def test_query_stream_w_order_by(query_docs):
 
 async def test_query_stream_w_field_path(query_docs):
     collection, stored, allowed_vals = query_docs
-    query = collection.where("stats.sum", ">", 4)
+    query = collection.where(filter=FieldFilter("stats.sum", ">", 4))
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     assert len(values) == 10
     ab_pairs2 = set()
@@ -685,7 +703,7 @@ async def test_query_stream_w_start_end_cursor(query_docs):
 async def test_query_stream_wo_results(query_docs):
     collection, stored, allowed_vals = query_docs
     num_vals = len(allowed_vals)
-    query = collection.where("b", "==", num_vals + 100)
+    query = collection.where(filter=FieldFilter("b", "==", num_vals + 100))
     values = [i async for i in query.stream()]
     assert len(values) == 0
 
@@ -693,7 +711,9 @@ async def test_query_stream_wo_results(query_docs):
 async def test_query_stream_w_projection(query_docs):
     collection, stored, allowed_vals = query_docs
     num_vals = len(allowed_vals)
-    query = collection.where("b", "<=", 1).select(["a", "stats.product"])
+    query = collection.where(filter=FieldFilter("b", "<=", 1)).select(
+        ["a", "stats.product"]
+    )
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     assert len(values) == num_vals * 2  # a ANY, b in (0, 1)
     for key, value in values.items():
@@ -706,7 +726,9 @@ async def test_query_stream_w_projection(query_docs):
 
 async def test_query_stream_w_multiple_filters(query_docs):
     collection, stored, allowed_vals = query_docs
-    query = collection.where("stats.product", ">", 5).where("stats.product", "<", 10)
+    query = collection.where(filter=FieldFilter("stats.product", ">", 5)).where(
+        "stats.product", "<", 10
+    )
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     matching_pairs = [
         (a_val, b_val)
@@ -725,7 +747,7 @@ async def test_query_stream_w_offset(query_docs):
     collection, stored, allowed_vals = query_docs
     num_vals = len(allowed_vals)
     offset = 3
-    query = collection.where("b", "==", 2).offset(offset)
+    query = collection.where(filter=FieldFilter("b", "==", 2)).offset(offset)
     values = {snapshot.id: snapshot.to_dict() async for snapshot in query.stream()}
     # NOTE: We don't check the ``a``-values, since that would require
     #       an ``order_by('a')``, which combined with the ``b == 2``
@@ -790,7 +812,7 @@ async def test_query_unary(client, cleanup):
     cleanup(document1.delete)
 
     # 0. Query for null.
-    query0 = collection.where(field_name, "==", None)
+    query0 = collection.where(filter=FieldFilter(field_name, "==", None))
     values0 = [i async for i in query0.stream()]
     assert len(values0) == 1
     snapshot0 = values0[0]
@@ -798,7 +820,7 @@ async def test_query_unary(client, cleanup):
     assert snapshot0.to_dict() == {field_name: None}
 
     # 1. Query for a NAN.
-    query1 = collection.where(field_name, "==", nan_val)
+    query1 = collection.where(filter=FieldFilter(field_name, "==", nan_val))
     values1 = [i async for i in query1.stream()]
     assert len(values1) == 1
     snapshot1 = values1[0]
@@ -907,10 +929,18 @@ async def test_collection_group_queries_filters(client, cleanup):
     query = (
         client.collection_group(collection_group)
         .where(
-            firestore.field_path.FieldPath.document_id(), ">=", client.document("a/b")
+            filter=FieldFilter(
+                firestore.field_path.FieldPath.document_id(),
+                ">=",
+                client.document("a/b"),
+            )
         )
         .where(
-            firestore.field_path.FieldPath.document_id(), "<=", client.document("a/b0")
+            filter=FieldFilter(
+                firestore.field_path.FieldPath.document_id(),
+                "<=",
+                client.document("a/b0"),
+            )
         )
     )
     snapshots = [i async for i in query.stream()]
@@ -920,12 +950,18 @@ async def test_collection_group_queries_filters(client, cleanup):
     query = (
         client.collection_group(collection_group)
         .where(
-            firestore.field_path.FieldPath.document_id(), ">", client.document("a/b")
+            filter=FieldFilter(
+                firestore.field_path.FieldPath.document_id(),
+                ">",
+                client.document("a/b"),
+            )
         )
         .where(
-            firestore.field_path.FieldPath.document_id(),
-            "<",
-            client.document("a/b/{}/cg-doc3".format(collection_group)),
+            filter=FieldFilter(
+                firestore.field_path.FieldPath.document_id(),
+                "<",
+                client.document("a/b/{}/cg-doc3".format(collection_group)),
+            )
         )
     )
     snapshots = [i async for i in query.stream()]
@@ -1551,7 +1587,7 @@ async def test_async_count_query_stream_empty_aggregation(async_query):
 @firestore.async_transactional
 async def create_in_transaction_helper(transaction, client, collection_id, cleanup):
     collection = client.collection(collection_id)
-    query = collection.where("a", "==", 1)
+    query = collection.where(filter=FieldFilter("a", "==", 1))
     count_query = query.count()
     result = await count_query.get(transaction=transaction)
 
@@ -1583,12 +1619,139 @@ async def test_count_query_in_transaction(client, cleanup):
 
     with pytest.raises(ValueError) as exc:
         await create_in_transaction_helper(transaction, client, collection_id, cleanup)
-        assert exc.exc_info == "Collection can't have more than 2 documents"
+    assert str(exc.value) == "Collection can't have more than 2 docs"
 
     collection = client.collection(collection_id)
 
-    query = collection.where("a", "==", 1)
+    query = collection.where(filter=FieldFilter("a", "==", 1))
     count_query = query.count()
     result = await count_query.get()
     for r in result[0]:
         assert r.value == 2  # there are still only 2 docs
+
+
+async def test_query_with_and_composite_filter(query_docs):
+    collection, stored, allowed_vals = query_docs
+    and_filter = And(
+        filters=[
+            FieldFilter("stats.product", ">", 5),
+            FieldFilter("stats.product", "<", 10),
+        ]
+    )
+
+    query = collection.where(filter=and_filter)
+    async for result in query.stream():
+        assert result.get("stats.product") > 5
+        assert result.get("stats.product") < 10
+
+
+async def test_query_with_or_composite_filter(query_docs):
+    collection, stored, allowed_vals = query_docs
+    or_filter = Or(
+        filters=[
+            FieldFilter("stats.product", ">", 5),
+            FieldFilter("stats.product", "<", 10),
+        ]
+    )
+    query = collection.where(filter=or_filter)
+    gt_5 = 0
+    lt_10 = 0
+    async for result in query.stream():
+        value = result.get("stats.product")
+        assert value > 5 or value < 10
+        if value > 5:
+            gt_5 += 1
+        if value < 10:
+            lt_10 += 1
+
+    assert gt_5 > 0
+    assert lt_10 > 0
+
+
+async def test_query_with_complex_composite_filter(query_docs):
+    collection, stored, allowed_vals = query_docs
+    field_filter = FieldFilter("b", "==", 0)
+    or_filter = Or(
+        filters=[FieldFilter("stats.sum", "==", 0), FieldFilter("stats.sum", "==", 4)]
+    )
+    # b == 0 && (stats.sum == 0 || stats.sum == 4)
+    query = collection.where(filter=field_filter).where(filter=or_filter)
+
+    sum_0 = 0
+    sum_4 = 0
+    async for result in query.stream():
+        assert result.get("b") == 0
+        assert result.get("stats.sum") == 0 or result.get("stats.sum") == 4
+        if result.get("stats.sum") == 0:
+            sum_0 += 1
+        if result.get("stats.sum") == 4:
+            sum_4 += 1
+
+    assert sum_0 > 0
+    assert sum_4 > 0
+
+    # b == 3 || (stats.sum == 4  && a == 4)
+    comp_filter = Or(
+        filters=[
+            FieldFilter("b", "==", 3),
+            And([FieldFilter("stats.sum", "==", 4), FieldFilter("a", "==", 4)]),
+        ]
+    )
+    query = collection.where(filter=comp_filter)
+
+    b_3 = False
+    b_not_3 = False
+    async for result in query.stream():
+        if result.get("b") == 3:
+            b_3 = True
+        else:
+            b_not_3 = True
+            assert result.get("stats.sum") == 4
+            assert result.get("a") == 4
+
+    assert b_3 is True
+    assert b_not_3 is True
+
+
+async def test_or_query_in_transaction(client, cleanup):
+    collection_id = "doc-create" + UNIQUE_RESOURCE_ID
+    document_id_1 = "doc1" + UNIQUE_RESOURCE_ID
+    document_id_2 = "doc2" + UNIQUE_RESOURCE_ID
+
+    document_1 = client.document(collection_id, document_id_1)
+    document_2 = client.document(collection_id, document_id_2)
+
+    cleanup(document_1.delete)
+    cleanup(document_2.delete)
+
+    await document_1.create({"a": 1, "b": 2})
+    await document_2.create({"a": 1, "b": 1})
+
+    transaction = client.transaction()
+
+    with pytest.raises(ValueError) as exc:
+        await create_in_transaction_helper(transaction, client, collection_id, cleanup)
+    assert str(exc.value) == "Collection can't have more than 2 docs"
+
+    collection = client.collection(collection_id)
+
+    query = collection.where(filter=FieldFilter("a", "==", 1)).where(
+        filter=Or([FieldFilter("b", "==", 1), FieldFilter("b", "==", 2)])
+    )
+    b_1 = False
+    b_2 = False
+    count = 0
+    async for result in query.stream():
+        assert result.get("a") == 1  # assert a==1 is True in both results
+        assert result.get("b") == 1 or result.get("b") == 2
+        if result.get("b") == 1:
+            b_1 = True
+        if result.get("b") == 2:
+            b_2 = True
+        count += 1
+
+    assert b_1 is True  # assert one of them is b == 1
+    assert b_2 is True  # assert one of them is b == 2
+    assert (
+        count == 2
+    )  # assert only 2 results, the third one was rolledback and not created
